@@ -1,5 +1,5 @@
 /* Easy Publish and Consume Library
- * Copyright (C) 2007  Openismus GmbH
+ * Copyright (C) 2007, 2008  Openismus GmbH
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,16 +19,15 @@
  *      Mathias Hasselmann
  */
 
-#include "consumer.h"
+#include "libepc/consumer.h"
 
-#include "enums.h"
-#include "marshal.h"
-#include "service-monitor.h"
-#include "shell.h"
+#include "libepc/enums.h"
+#include "libepc/i18n.h"
+#include "libepc/marshal.h"
+#include "libepc/service-monitor.h"
+#include "libepc/shell.h"
 
-#include <glib/gi18n-lib.h>
-#include <libsoup/soup-session-async.h>
-#include <libsoup/soup-uri.h>
+#include <libsoup/soup.h>
 #include <string.h>
 
 /**
@@ -174,6 +173,8 @@ static guint signals[SIGNAL_LAST];
 
 G_DEFINE_TYPE (EpcConsumer, epc_consumer, G_TYPE_OBJECT);
 
+#ifdef HAVE_LIBSOUP22
+
 static void
 epc_consumer_authenticate_cb (SoupSession  *session G_GNUC_UNUSED,
                               SoupMessage  *message,
@@ -232,6 +233,54 @@ epc_consumer_reauthenticate_cb (SoupSession  *session,
                                   auth_type, username, password, data);
 }
 
+#else
+
+static void
+epc_consumer_authenticate_cb (SoupSession  *session G_GNUC_UNUSED,
+                              SoupMessage  *message,
+                              SoupAuth     *auth,
+                              gboolean      retrying,
+                              gpointer      data)
+{
+  EpcConsumer *self = EPC_CONSUMER (data);
+  const char *username, *password;
+  gboolean handled = FALSE;
+
+  if (EPC_DEBUG_LEVEL (1))
+    g_debug ("%s: path=%s, realm=%s, retrying=%d",
+             G_STRLOC, soup_message_get_uri (message)->path,
+             soup_auth_get_realm (auth), retrying);
+
+  if (retrying)
+    {
+      g_signal_emit (self, signals[SIGNAL_AUTHENTICATE],
+                     0, soup_auth_get_realm (auth), &handled);
+
+      if (EPC_DEBUG_LEVEL (1))
+        g_debug ("%s: path=%s, realm=%s, handled=%d",
+                 G_STRLOC, soup_message_get_uri (message)->path,
+                 soup_auth_get_realm (auth), handled);
+    }
+  else
+    handled = TRUE;
+
+  if (handled)
+    {
+      username = (self->priv->username ? self->priv->username : "");
+      password = (self->priv->password ? self->priv->password : "");
+
+      soup_auth_authenticate (auth, username, password);
+
+      if (EPC_DEBUG_LEVEL (1))
+        g_debug ("%s: path=%s, realm=%s, retrying=%d, username=%s, password=%s",
+                 G_STRLOC, soup_message_get_uri (message)->path,
+                 soup_auth_get_realm (auth), retrying,
+                 username, password);
+    }
+}
+
+#endif
+
 static void
 epc_consumer_init (EpcConsumer *self)
 {
@@ -241,8 +290,10 @@ epc_consumer_init (EpcConsumer *self)
 
   g_signal_connect (self->priv->session, "authenticate",
                     G_CALLBACK (epc_consumer_authenticate_cb), self);
+#ifdef HAVE_LIBSOUP22
   g_signal_connect (self->priv->session, "reauthenticate",
                     G_CALLBACK (epc_consumer_reauthenticate_cb), self);
+#endif
 }
 
 static void
@@ -577,15 +628,15 @@ epc_consumer_class_init (EpcConsumerClass *cls)
    * @port:  the publisher's TCP/IP port
    *
    * This signal is emitted when a #EpcConsumer created with
-   * #epc_consumer_new_for_name or #epc_consumer_new_for_name_full
+   * epc_consumer_new_for_name() or #epc_consumer_new_for_name_full
    * has found its #EpcPublisher.
    *
    * Publisher detection is integrated with the GLib main loop. Therefore the
-   * signal will not be emitted before a main loop is run (#g_main_loop_run,
-   * #gtk_main). So to reliably consume this signal connect to it directly
+   * signal will not be emitted before a main loop is run (g_main_loop_run(),
+   * gtk_main()). So to reliably consume this signal connect to it directly
    * after creating the #EpcConsumer.
    *
-   * See also: #epc_consumer_resolve_publisher, #epc_consumer_is_pulisher_resolved
+   * See also: epc_consumer_resolve_publisher(), #epc_consumer_is_pulisher_resolved
    */
   signals[SIGNAL_PUBLISHER_RESOLVED] = g_signal_new ("publisher-resolved", EPC_TYPE_CONSUMER, G_SIGNAL_RUN_FIRST,
                                                      G_STRUCT_OFFSET (EpcConsumerClass, publisher_resolved), NULL, NULL,
@@ -604,8 +655,8 @@ epc_consumer_class_init (EpcConsumerClass *cls)
  * by using #EpcServiceMonitor, or by using the service selection dialog
  * of <citetitle>avahi-ui</citetitle> (#AuiServiceDialog).
  *
- * The connection is not established until functions like #epc_consumer_lookup,
- * #epc_consumer_list or #epc_consumer_resolve_publisher are called.
+ * The connection is not established until functions like epc_consumer_lookup(),
+ * epc_consumer_list() or #epc_consumer_resolve_publisher are called.
  *
  * Returns: The newly created #EpcConsumer object
  */
@@ -637,9 +688,9 @@ epc_consumer_new (const EpcServiceInfo *service)
  * Creates a new #EpcConsumer object and associates it with the #EpcPublisher
  * announcing itself with @name on the local network. The DNS-SD service name
  * used for searching the #EpcPublisher is derived from the application's
- * program name as returned by #g_get_prgname.
+ * program name as returned by g_get_prgname().
  *
- * See #epc_consumer_new_for_name_full for additional notes
+ * See epc_consumer_new_for_name_full() for additional notes
  * and a method allowing better control over the search process.
  *
  * Returns: The newly created #EpcConsumer object
@@ -658,12 +709,12 @@ epc_consumer_new_for_name (const gchar *name)
  *
  * Creates a new #EpcConsumer object and associates it with the #EpcPublisher
  * announcing itself with @name on @domain. The DNS-SD service of the
- * #EpcPublisher is derived from @application using #epc_service_type_new.
+ * #EpcPublisher is derived from @application using epc_service_type_new().
  *
  * <note><para>
  *  This function shall be used to re-connect to a formerly used #EpcPublisher,
  *  selected for instance from a list for recently used services. Therefore
- *  using #epc_consumer_new_for_name_full is a quite optimistic approach for
+ *  using epc_consumer_new_for_name_full() is a quite optimistic approach for
  *  contacting a publisher: You call it without really knowing if the
  *  publisher you requested really exists. You only know that it existed
  *  in the past when you added it to your list of recently used publishers,
@@ -672,14 +723,14 @@ epc_consumer_new_for_name (const gchar *name)
  *  To let your users choose from an up-to-date service list, you have to
  *  use a dynamic service list as provided by avahi-ui for choosing a service
  *  and pass the information this widget provides (hostname, port, protocol)
- *  to #epc_consumer_new.
+ *  to epc_consumer_new().
  * </para></note>
  *
  * <note><para>
  *  The connection is not established until a function retrieving
- *  data, like for instance #epc_consumer_lookup, is called.
+ *  data, like for instance epc_consumer_lookup(), is called.
  *
- *  Explicitly call #epc_consumer_resolve_publisher or connect to
+ *  Explicitly call epc_consumer_resolve_publisher() or connect to
  *  the #EpcConsumer::publisher-resolved signal, when your application
  *  needs reliable information about the existance of the #EpcPublisher
  *  described by @name.
@@ -843,7 +894,7 @@ epc_consumer_resolve_publisher (EpcConsumer *self,
  * Checks if the host name of this consumer's #EpcPublisher
  * has been resolved already.
  *
- * See also: #epc_consumer_resolve_publisher, #EpcPublisher::publisher-resolved
+ * See also: epc_consumer_resolve_publisher(), #EpcPublisher::publisher-resolved
  *
  * Returns: %TRUE when the host name has been resolved, and %FALSE otherwise.
  */
@@ -916,7 +967,7 @@ epc_consumer_set_http_error (GError     **error,
  * enumeration.
  *
  * For instance, the error code will be #SOUP_STATUS_FORBIDDEN if
- * authentication failed (see #epc_publisher_set_auth_handler). You must
+ * authentication failed (see epc_publisher_set_auth_handler()). You must
  * include <filename class="headerfile">libsoup/soup-status.h</filename>
  * to use this error code.
  *
@@ -960,15 +1011,21 @@ epc_consumer_lookup (EpcConsumer  *self,
 
   if (SOUP_STATUS_IS_SUCCESSFUL (status))
     {
+#ifdef HAVE_LIBSOUP22
+      const gsize response_length = request->response.length;
+      gconstpointer response_data = request->response.body;
+#else
+      const gsize response_length = request->response_body->length;
+      gconstpointer response_data = request->response_body->data;
+#endif
+
       if (length)
-        *length = request->response.length;
+        *length = response_length;
 
-      contents = g_malloc (request->response.length + 1);
-      contents[request->response.length] = '\0';
+      contents = g_malloc (response_length + 1);
+      contents[response_length] = '\0';
 
-      memcpy (contents,
-              request->response.body,
-              request->response.length);
+      memcpy (contents, response_data, response_length);
     }
   else
     epc_consumer_set_http_error (error, request, status);
@@ -1019,7 +1076,7 @@ epc_consumer_list_parser_start_element (GMarkupParseContext *context G_GNUC_UNUS
   else
     g_set_error (error, G_MARKUP_ERROR,
                  G_MARKUP_ERROR_INVALID_CONTENT,
-                 _("Unexpected element: `%s'"),
+                 _("Unexpected element: '%s'"),
                 element_name);
 }
 
@@ -1101,7 +1158,7 @@ epc_consumer_list_parser_text (GMarkupParseContext *context G_GNUC_UNUSED,
  *  g_list_free (keys);
  * </programlisting>
  *
- * See also #epc_publisher_list for creating custom listings.
+ * See also epc_publisher_list() for creating custom listings.
  *
  * Returns: A newly allocated list of keys, or %NULL when an error occurred.
  */
@@ -1146,16 +1203,26 @@ epc_consumer_list (EpcConsumer  *self,
                                             G_MARKUP_TREAT_CDATA_AS_TEXT,
                                             &state, NULL);
 
+#ifdef HAVE_LIBSOUP22
       g_markup_parse_context_parse (context,
                                     request->response.body,
                                     request->response.length,
                                     error);
+#else
+      g_markup_parse_context_parse (context,
+                                    request->response_body->data,
+                                    request->response_body->length,
+                                    error);
+#endif
 
       g_markup_parse_context_free (context);
     }
   else
     epc_consumer_set_http_error (error, request, status);
 
+  if (request)
+    g_object_unref (request);
+
   return state.items;
 }
 
