@@ -1,5 +1,5 @@
 /* Easy Publish and Consume Library
- * Copyright (C) 2007  Openismus GmbH
+ * Copyright (C) 2007, 2008  Openismus GmbH
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,12 +19,12 @@
  *      Mathias Hasselmann
  */
 
-#include "dispatcher.h"
+#include "libepc/dispatcher.h"
 
-#include "enums.h"
-#include "service-monitor.h"
-#include "service-type.h"
-#include "shell.h"
+#include "libepc/enums.h"
+#include "libepc/service-monitor.h"
+#include "libepc/service-type.h"
+#include "libepc/shell.h"
 
 #include <avahi-common/alternative.h>
 #include <avahi-common/error.h>
